﻿# Listing 17.1

Save-Help Update-Help -Examples

# Listing 17.2
Get-Executionpolicy

# Listing 17.3
PowerShell.exe -exectutionpolicy bypass -file \\server1\skripte\config.ps1

# Listing 17.4
Get-ExecutionPolicy -List

# Listing 17.5
Get-GPO -Name "Default Domain Policy"

# Listing 17.6
Get-Gpo -All | Export-CSV -Path "$env:temp\AllePolicies.CSV" -Delimiter ";"

# Listing 17.7
Get-GpoReport -Name 'Default Domain Policy' -path 'C:\GpoReport\DefaultDomainPolicy.hta' -ReportType Html

# Listing 17.8
Get-Gpo -All | ForEach-Object {
  $path = 'C:\GPOResult\' + $_.DisplayName + '.hta'
  Get-GPOReport -Name $_.DisplayName -Path $path -ReportType Html }
 
# Listing 17.9
Backup-GPO -All -Path C:\GPOBackup -Comment "Automatisch generiertes Backup"

# Listing 17.10
$TimeStamp = Get-Date -Format ddMMyyyy_HHmmss
Backup-GPO -All -Path C:\GPOBackup -Comment "Automatisch generiertes Backup" |
  Export-CSV -Path C:\GPOBackup\gpoBackup$TimeStamp.CSV -Delimiter ";"

# Listing 17.11
Get-ChildItem -Path c:\GPOBackup\*.CSV | Select-Object -ExpandProperty FullName | >
Import-CSV Delimiter ";"

# Listing 17.12
Get-ChildItem -path *.CSV | Select-Object -ExpandProperty FullName |
import-CSV Delimiter ";" | Out-GridView -PassThru | Restore-GPO -Confirm

# Listing 17.13 - nur geänderte GPOs anzeigen
$now = Get-Date
Get-GPO -All | Where-Object { $_.ModificationTime -gt $now.AddDays(-1) }

# Listing 17.14 Einen Einblick in ein Objekt liefert get-member
$now | get-member -MemberType Method

# Listing 17.15 Sichern aller kürzlich modifizierten GPOs
Get-GPO -All | Where-Object { $_.ModificationTime -gt $now.AddDays(-1) } |
  Backup-GPO -Path C:\GPOBackup -Comment "Automatisch generiertes Backup" |
  Export-CSV -Path C:\GPOBackup\gpoBackup$TimeStamp.CSV -Delimiter ";"

# Listing 17.16 Lange nicht modifizierte GPOs anzeigen
$now = Get-Date
Get-GPO -All | Where-Object { $_.ModificationTime -lt $now.AddDays(-365) }

# Listing 17.17 Richtlinien mit WMI-Filtern auflisten
Get-GPO -all | Where-Object { $_.WMIFilter }

# Listing 17.18 Richtlinien ohne Beschreibung auflisten
Get-GPO -All | Where-Object { -not $_.Description }

# Listing 17.19 Alle Eigenschaften einer GPO anzeigen mit Select-Object *
Get-GPo -Name "Default Domain Policy" | Select-Object *

# Listing 17.20 Das Unterobjekt der Eigenschaft User anzeigen
Get-GPo -Name "Default Domain Policy" | Select-Object -ExpandProperty User

# Listing 17.21 Der XML-Report als PowerShell-Objekt
[xml]$gpoReport = Get-GPOReport -ReportType Xml -name "default Domain Policy"
$gpoReport

# Listing 17.22 Die Eigenschaften der Eigenschaft gpo
$gpoReport.gpo

# Listing 17.23 ExtensionData enthält die Einstellungen der Richtlinie.
$gpoReport.gpo.Computer.ExtensionData

# Listing 17.24 Get-EmptyGPO findet leere GPOs über den XML-Report.
function Get-EmptyGPO {
    param
    (
        # Name des zu untersuchenden GPOs
        [string]$name,   
        # Der Bereich, der untersucht werden soll
        [Validateset('All','NoComputerSettings','NoUsersettings')]
        [String]$Scope = 'All'
    )
  # Den XML-Report der Gruppenrichtlinie erzeugen
  [xml]$gpoReport = Get-GPOReport -ReportType Xml -name $name
  Switch ($Scope)
  {
    # esten auf nicht existierenden user.Extensiondata-Knoten
    'NoUserSettings' 
    {
        If ((-not $gpoReport.gpo.user.extensiondata) -And ($gpoReport.gpo.computer.extensiondata))
           { Get-GPO -Name $name }                
    }
    #  Testen auf nicht existierenden Computer-Extensiondata-Knoten
    'NoComputerSettings' 
    {
        If (($gpoReport.gpo.user.extensiondata) -And (-not $gpoReport.gpo.computer.extensiondata))
           { Get-GPO -Name $name }   
    }
    'All' 
    { 
        If ((-not $gpoReport.gpo.user.extensiondata) -And (-not $gpoReport.gpo.computer.extensiondata))
           { Get-GPO -Name $name }
    }
}} 

# Listing 17.26 Switch ist ein übersichtliches IF.
$toTest = "wahr"
SWITCH ( $toTest )
{
    {$_ -eq "wahr"} { "Ehrlich währt am längsten" }
    {$_ -eq "falsch"} { "Lügen haben kurze Beine" }
    {$_ -eq "halbwahr"} { "Die halbe Wahrheit ist auch eine Wahrheit" }
}

# Listing 17.27 Wenn immer nur auf wahr geprüft wird, kann Switch verkürzt werden.
$toTest = "wahr"
SWITCH ( $toTest )
{
    "wahr"     { "Ehrlich währt am längsten" }
    "falsch"   { "Lügen haben kurze Beine" }
    "halbwahr" { "Die halbe Wahrheit ist auch eine Wahrheit" }
}

# Listing 17.28 Alle GPOs darauf prüfen, ob sie komplett leer sind
Get-GPO -ALL | Get-EmtpyGPO

# Listing 17.29 Die verknüpften GPOs auf der Domäne auflisten
Get-ADDomain | Select-Object LinkedGroupPolicyObjects

# Listing 17.30 Die verknüpften GPOs auf allen OUs der Domäne auflisten
Get-ADOrganizationalUnit -Filter * | Select-Object LinkedGroupPolicyObject

# Listing 17.31 Mit der Methode Matches kann man Zeichenmuster extrahieren.
[regex]$GUID = '[A-Fa-f\d]{8}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa- f\d]{12}'
$guid.matches(( Get-ADDomain ).LinkedGroupPolicyObjects)

# Listing 17.32 Die Liste aller verknüpften GPOs der Domäne
[regex]$GUID = '[A-Fa-f\d]{8}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{12}'
$Policies = @( $guid.matches(( Get-ADDomain ).LinkedGroupPolicyObjects).Value )
$Policies += @( $guid.matches(( Get-ADOrganizationalUnit -filter *.LinkedGroupPolicyObjects).Value )

# Listing 17.33 Die Variable $Policies enthält die GUIDs aller verknüpften GPOs.
$policies

# Listing 17.34 Nicht verknüpfte GPOs auflisten
[regex]$GUID = '[A-Fa-f\d]{8}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{12}'
$Policies = @( $guid.matches(( Get-ADDomain ).LinkedGroupPolicyObjects).Value )
$Policies += @( $guid.matches(( Get-ADOrganizationalUnit -filter *.LinkedGroupPolicyObjects).Value )
Get-GPO -all | Where-Object { $_.id -notin $Policies }

# Listing 17.35 Die Registry-Einstellungen aus der Registry.Pol-Datei auslesen
Get-GPRegistryValue -Name "RDP Server" -Key "HKCU\SOFTWARE\Policies\Microsoft\Windows\System" -Value "DisableCMD"

# Listing 17.36 Alle Gruppenrichtlinienobjekte mit einer bestimmten Einstellung finden
regKey = "HKCU\SOFTWARE\Policies\Microsoft\Windows\System"
$regValueName = 'DisableCMD'
Get-GPO -all | ForEach-Object {
  $Setting = Get-GPRegistryValue -name $_.DisplayName -Key $regKey -ValueName $regValueName -ErrorAction SilentlyContinue
  IF ( $Setting )
  {
    $_.DisplayName
    $setting
  }
}

# Listing 17.37 Alle Richtlinien mit einer Einstellung finden, V 2.0
Get-GPO -all | ForEach-Object {
  $Setting = Get-GPRegistryValue -name $_.DisplayName -Key $regKey -ValueName $regValueName -ErrorAction SilentlyContinue
  IF ( $Setting )
  {
    Add-Member -InputObject $setting -NotepropertyName gpoName -NotePropertyValue -$_.Displayname
    $setting | select *
  }
} | Out-GridView

# Listing 17.38 Die Default Domain Policy abrufen
Get-ADDefaultDomainPasswordPolicy

# Listing 17.39 Granulierte Kennwortrichtlinien abrufen
Get-ADFineGrainedPasswordPolicy -Filter *

# Listing 17.40 WMI-Filter auflisten
Get-ADObject -Filter 'objectClass -eq "msWMI-Som"' -Properties msWMIName,msWMIParm2,Created,Modified,msWMI-Author

# Listing 17.41 WMI-Filter sichern
Get-ADObject -Filter 'objectClass -eq "msWMI-Som"' -Properties 'msWMI-Name', 'msWMI-Parm1','msWMI-Parm2' |
    Select-Object DistinguishedName, msWMI-Name, msWMI-Parm1, msWMI-Parm2, Name |
    Export-CSV -NoTypeInformation -Encoding UTF8 C:\gpobackup\WMIFilter.CSV
 
# Listing 17.42 WMI-Filter eines Gruppenrichtlinienobjekts anzeigen
get-gpo "RDP Server"

# Listing 17.43 Die Abfrage eines WMI-Filters aus dem AD auslesen
$WMIFlt = Get-GPO "RDP Server" | Select-Object -ExpandProperty WMIFilter
$WMIFlt.GetQueryList()

# Listing 17.44 Alle WMI-Filter mit Richtliniennamen in ein GridView ausgeben
Get-GPO -All | Where-Object { $_.WMIFilter } |
  ForEach-Object {
    Add-Member -InputObject $_.WMIFilter -NotePropertyName gpoName -NotePropertyValue $_.Displayname
    # Die Ausgabe ist wichtig, damit der Filter auch angezeigt wird!
    $_.WmiFilter
  } | out-Gridview

# Listing 17.45 Einen neuen Registry-Wert per Set-GPRegistryValue setzen
Set-GPRegistryValue -Name 'IE keine Hilfe' -key 'HKLM\Software\Policies\Microsoft\Internet Explorer\Restrictions' -Valuename 'NoHelpMenu' -Type DWord -Value 1

# Listing 17.46 Registry-basierte GPO-Einstellungen per PowerShell setzen
Set-GPPrefRegistryValue -Name "RDP Server" -Context User -Action Create -Key "HKEY_Current_User\Software\Policies\Microsoft\Office\16.0\Common\General" -ValueName shownfirstrunoptin -Value 1 -Type DWord
Set-GPPrefRegistryValue -Name "RDP Server" -Context User -Action Create -Key "HKEY_Current_User\Software\Policies\Microsoft\Office\16.0\Common\General" -ValueName ShownFileFmtPrompt -Value 1 -Type DWord

# Listing 17.47 Setzen der GPO-Berechtigungen über PowerShell
Set-GPPermission -Name 'IE keine Hilfe' -Replace -Permissionlevel None -Targetname 'Authenticated Users' -Targettype group
Set-GPPermission -Name 'IE keine Hilfe' -Permissionlevel gpoapply -TargetName 'Userhelpdesk' -Targettype group
Set-GPPermission -Name 'IE keine Hilfe' -Permissionlevel GpoRead -TargetName 'Domänencomputer' -Targettype group

# Listing 17.48 GPOs verknüpfen mit New-GPLink
New-GPLink -Name 'IE keine Hilfe' -Target "OU=Hannover,dc=bit-weise,dc=de" -Order 1

# Listing 17.49 Abfragen der Vererbungsinformationen für die OU Domain Controllers
Get-GPInheritance -Target "OU=Domain Controllers,dc=forschung,dc=bit-weise,dc=de"

# Listing 17.50 Gruppenrichtlinienobjekt-Berechtigungen abfragen
Get-GPPermission -Name "Default Domain Policy" -All

# Listing 17.51 Alle Berechtigungen eines Gruppenrichtlinienobjekts auflisten
function Get-GPPermissionListExtended
{
  param(
    [parameter(ValueFromPipeline=$true)]
    # GPO-Object for Permission-Analysis
    $gpo
  )

  process {
    $gpoName = $Gpo.displayname
    $gppermissions = Get-GPPermission -Guid $gpo.ID -All
    foreach ( $gppermission in $gppermissions )
    {
      Add-Member -InputObject $gppermission -NotePropertyName GPOName -NotePropertyValue $gpoName
      Add-Member -InputObject $gppermission -NotePropertyName Domain -NotePropertyValue $gppermission.Trustee.Domain
      Add-Member -InputObject $gppermission -NotePropertyName DSPath -NotePropertyValue $gppermission.Trustee.DSPath
      Add-Member -InputObject $gppermission -NotePropertyName Name -NotePropertyValue $gppermission.Trustee.Name
      Add-Member -InputObject $gppermission -NotePropertyName SID -NotePropertyValue $gppermission.Trustee.SID
      Add-Member -InputObject $gppermission -NotePropertyName SIDType -NotePropertyValue $gppermission.Trustee.SIDType
      $gppermission | select *
    }
}}
get-gpo -name 'Default Domain Policy' | Get-GPPermissionListExtended | out-Gridview

# Listing 17.52 Ein RSOP mit PowerShell erstellen
Get-GPResultantSetOfPolicy -Computer Win10Enterprise -User bit-weise\hwurst -ReportType html -Path C:\temp\hwurstRep.hta

# Listing 17.53 Eine neue Gruppenrichtlinienverknüpfung anlegen
New-GPLink -Name "RDP Server" -Target "ou=Benutzer,ou=Hannover,dc=bit-weise,dc=de" -Order 1 -LinkEnabled yes -Enforced no

# Listing 17.54 Eine Gruppenrichtlinienverknüpfung ändern
Set-GPLink -Name "RDP Server" -Target "ou=Benutzer,ou=Hannover,dc=bit-weise,dc=de" -LinkEnabled no -enforced no

# Listing 17.55 Entfernen einer Gruppenrichtlinienverknüpfung
Remove-GPLink -Name "RDP Server" -Target "ou=Benutzer,ou=Hannover,dc=bit-weise,dc=de"

# Listing 17.56 Die Sysvol-Version sollte mit der AD-Version übereinstimmen.
Get-ADGPOReplication -All

# Listing 17.57 Die AppLocker-Skriptbeschränkung mit PowerShell umgehen
Get-Content .\meinBoesesScript.txt | PowerShell.exe -Command -

# Listing 17.58 Abfragen des PowerShell-Sprachmodus
$ExecutionContext.SessionState.LanguageMode